@@ -13,11 +13,18 @@
 #include <cstring>
 #include <optional>
 
-#include <dirent.h>
 #include <sys/stat.h>
 
 #include <ancient/ancient.hpp>
 
+// enables scanning. Useful for testing/debugging. Not so useful otherwise
+// Not good for default since creates dependency for dirent (i.e. windows blues)
+//#define ENABLE_SCAN 1
+
+#ifdef ENABLE_SCAN
+#include <dirent.h>
+#endif
+
 std::unique_ptr<std::vector<uint8_t>> readFile(const std::string &fileName)
 {
 	std::unique_ptr<std::vector<uint8_t>> ret=std::make_unique<std::vector<uint8_t>>();
@@ -67,15 +74,20 @@ int main(int argc,char **argv)
 {
 	auto usage=[]()
 	{
-		fprintf(stderr,"Usage: ancient identify packed_input_file\n");
-		fprintf(stderr," - identifies compression used in a file\n");
-		fprintf(stderr,"Usage: ancient verify packed_input_file unpacked_comparison_file\n");
-		fprintf(stderr," - verifies decompression against known good unpacked file\n");
-		fprintf(stderr,"Usage: ancient decompress packed_input_file output_file\n");
-		fprintf(stderr," - decompresses single file\n");
-		fprintf(stderr,"Usage: ancient scan input_dir output_dir\n");
-		fprintf(stderr," - scans input directory recursively and stores all found\n"
-			       " - known compressed streams to separate files in output directory\n");
+		fprintf(stderr, "Ancient v2.2.0\n"
+				"Copyright (C) Teemu Suutari\n"
+				"\n"
+				"Usage: ancient i[dentify] packed_input_files...\n"
+				" - identifies compression used in a file(s)\n"
+				"Usage: ancient v[erify] packed_input_file unpacked_comparison_file\n"
+				" - verifies decompression against known good unpacked file\n"
+				"Usage: ancient d[ecompress] packed_input_file output_file\n"
+				" - decompresses single file\n");
+#ifdef ENABLE_SCAN
+		fprintf(stderr,	"Usage: ancient s[can] input_dir output_dir\n"
+				" - scans input directory recursively and stores all found\n"
+				" - known compressed streams to separate files in output directory\n");
+#endif
 	};
 
 	if (argc<3)
@@ -85,26 +97,31 @@ int main(int argc,char **argv)
 	}
 	std::string cmd=argv[1];
 
-	if (cmd=="identify")
+	if (cmd=="i" || cmd=="identify")
 	{
-		if (argc!=3)
+		if (argc<3)
 		{
 			usage();
 			return -1;
 		}
-		auto packed{readFile(argv[2])};
-		std::optional<ancient::Decompressor> decompressor;
-		try
+		for (int i=2;i<argc;i++)
 		{
-			decompressor.emplace(*packed,true,true);
-			printf("Compression of %s is %s\n",argv[2],decompressor->getName().c_str());
-		} catch (const ancient::InvalidFormatError&)
-		{
-			fprintf(stderr,"Unknown or invalid compression format in file %s\n",argv[2]);
-			return -1;
+			auto packed{readFile(argv[i])};
+			std::optional<ancient::Decompressor> decompressor;
+			try
+			{
+				decompressor.emplace(*packed,true,true);
+				printf("Compression of %s is %s\n",argv[i],decompressor->getName().c_str());
+			} catch (const ancient::InvalidFormatError&)
+			{
+				fprintf(stderr,"Unknown or invalid compression format in file %s\n",argv[i]);
+			} catch (const ancient::VerificationError&)
+			{
+				fprintf(stderr,"Failed to validate file %s\n",argv[i]);
+			}
 		}
 		return 0;
-	} else if (cmd=="decompress" || cmd=="verify") {
+	} else if (cmd=="d" || cmd=="decompress" || cmd=="v" || cmd=="verify") {
 		if (argc!=4)
 		{
 			usage();
@@ -142,7 +159,7 @@ int main(int argc,char **argv)
 			return -1;
 		}
 
-		if (cmd=="decompress")
+		if (cmd=="d" || cmd=="decompress")
 		{
 			if (decompressor->getImageOffset() || decompressor->getImageSize())
 			{
@@ -178,7 +195,9 @@ int main(int argc,char **argv)
 			printf("Files match!\n");
 			return 0;
 		}
-	} else if (cmd=="scan") {
+	}
+#ifdef ENABLE_SCAN
+	else if (cmd=="s" || cmd=="scan") {
 		if (argc!=4)
 		{
 			usage();
@@ -210,54 +229,43 @@ int main(int argc,char **argv)
 						processDir(name);
 					} else if (st.st_mode&S_IFREG) {
 						auto packed{readFile(name)};
-						size_t scanPos=0;
-						size_t scanSize=packed->size();
 						for (size_t i=0;i<packed->size();)
 						{
-							scanPos+=1;
-							scanSize-=1;
 							// We will detect first, before trying the format for real
-							if (!ancient::Decompressor::detect(packed->data()+scanPos,scanSize))
+							if (!ancient::Decompressor::detect(packed->data()+i,packed->size()-i))
 							{
 								i++;
 								continue;
 							}
 							try
 							{
-								ancient::Decompressor decompressor{packed->data()+scanPos,scanSize,false,true};
-								// for formats that do not encode packed size.
-								// we will get it from decompressor
+								ancient::Decompressor decompressor{packed->data()+i,packed->size()-i,false,true};
+
+								printf("trying %s\n",decompressor.getName().c_str());
 								if (!decompressor.getPackedSize())
 								{
-									try
-									{
-										decompressor.decompress(true);
-									} catch (const std::bad_alloc&) {
-										fprintf(stderr,"Out of memory\n");
-										i++;
-										continue;
-									}
+									// for formats that do not encode packed size.
+									// we will get it from decompressor
+									decompressor.decompress(true);
 								}
-								if (decompressor.getPackedSize())
+
+								// final checks with the limited buffer and fresh decompressor
+								const uint8_t *finalData=packed->data()+i;
+								size_t finalSize=decompressor.getPackedSize().value();
+								ancient::Decompressor decompressor2{finalData,finalSize,true,true};
+								try
 								{
-									// final checks with the limited buffer and fresh decompressor
-									const uint8_t *finalData=packed->data()+i;
-									size_t finalSize=decompressor.getPackedSize().value();
-									ancient::Decompressor decompressor2{finalData,finalSize,true,true};
-									try
-									{
-										decompressor2.decompress(true);
-									} catch (const std::bad_alloc&) {
-										fprintf(stderr,"Out of memory\n");
-										i++;
-										continue;
-									}
-									std::string outputName=std::string(argv[3])+"/file"+std::to_string(fileIndex++)+".pack";
-									printf("Found compressed stream at %zu, size %zu in file %s with type '%s', storing it into %s\n",i,decompressor2.getPackedSize().value(),name.c_str(),decompressor2.getName().c_str(),outputName.c_str());
-									writeFile(outputName,finalData,finalSize);
-									i+=finalSize;
+									decompressor2.decompress(true);
+								} catch (const std::bad_alloc&) {
+									fprintf(stderr,"Out of memory\n");
+									i++;
 									continue;
 								}
+								std::string outputName=std::string(argv[3])+"/file"+std::to_string(fileIndex++)+".pack";
+								printf("Found compressed stream at %zu, size %zu in file %s with type '%s', storing it into %s\n",i,decompressor2.getPackedSize().value(),name.c_str(),decompressor2.getName().c_str(),outputName.c_str());
+								writeFile(outputName,finalData,finalSize);
+								i+=finalSize;
+								continue;
 							} catch (const ancient::Error&) {
 								// full steam ahead (with next offset)
 							} catch (const std::bad_alloc&) {
@@ -274,7 +282,9 @@ int main(int argc,char **argv)
 
 		processDir(std::string(argv[2]));
 		return 0;
-	} else {
+	}
+#endif
+	else {
 		fprintf(stderr,"Unknown command\n");
 		usage();
 		return -1;
