@@ -11,6 +11,7 @@
 #include "common/CRC32.hpp"
 #include "common/Common.hpp"
 
+#include <array>
 
 namespace ancient::internal
 {
@@ -36,38 +37,35 @@ std::shared_ptr<XPKDecompressor> BZIP2Decompressor::create(uint32_t hdr,uint32_t
 }
 
 BZIP2Decompressor::BZIP2Decompressor(const Buffer &packedData,bool exactSizeKnown,bool verify) :
-	_packedData(packedData),
-	_packedSize(0)
+	_packedData{packedData},
+	_packedSize{0}
 {
 	uint32_t hdr=packedData.readBE32(0);
-	if (!detectHeader(hdr)) throw Decompressor::InvalidFormatError();;
+	if (!detectHeader(hdr))
+		throw Decompressor::InvalidFormatError();;
 	_blockSize=((hdr&0xffU)-'0')*100'000;
 }
 
 BZIP2Decompressor::BZIP2Decompressor(uint32_t hdr,uint32_t recursionLevel,const Buffer &packedData,std::shared_ptr<XPKDecompressor::State> &state,bool verify) :
-	XPKDecompressor(recursionLevel),
-	_packedData(packedData),
-	_packedSize(_packedData.size())
+	XPKDecompressor{recursionLevel},
+	_packedData{packedData},
+	_packedSize{_packedData.size()}
 {
 	uint32_t blockHdr=packedData.readBE32(0);
-	if (!detectHeader(blockHdr)) throw Decompressor::InvalidFormatError();;
+	if (!detectHeader(blockHdr))
+		throw Decompressor::InvalidFormatError();;
 	_blockSize=((blockHdr&0xffU)-'0')*100'000;
 }
 
-BZIP2Decompressor::~BZIP2Decompressor()
-{
-	// nothing needed
-}
-
 const std::string &BZIP2Decompressor::getName() const noexcept
 {
-	static std::string name="bz2: bzip2";
+	static std::string name{"bz2: bzip2"};
 	return name;
 }
 
 const std::string &BZIP2Decompressor::getSubName() const noexcept
 {
-	static std::string name="XPK-BZP2: bzip2";
+	static std::string name{"XPK-BZP2: bzip2"};
 	return name;
 }
 
@@ -87,10 +85,9 @@ size_t BZIP2Decompressor::getRawSize() const noexcept
 void BZIP2Decompressor::decompressImpl(Buffer &rawData,bool verify)
 {
 	size_t packedSize=_packedSize?_packedSize:_packedData.size();
-	size_t rawSize=_rawSize?_rawSize:rawData.size();
 
-	ForwardInputStream inputStream(_packedData,4,packedSize);
-	MSBBitReader<ForwardInputStream> bitReader(inputStream);
+	ForwardInputStream inputStream{_packedData,4,packedSize};
+	MSBBitReader<ForwardInputStream> bitReader{inputStream};
 	auto readBits=[&](uint32_t count)->uint32_t
 	{
 		return bitReader.readBits8(count);
@@ -100,7 +97,7 @@ void BZIP2Decompressor::decompressImpl(Buffer &rawData,bool verify)
 		return bitReader.readBits8(1);
 	};
 
-	ForwardOutputStream outputStream(rawData,0,rawSize);
+	AutoExpandingForwardOutputStream outputStream{rawData};
 
 	// stream verification
 	//
@@ -120,23 +117,22 @@ void BZIP2Decompressor::decompressImpl(Buffer &rawData,bool verify)
 	HuffmanDecoder<uint8_t> selectorDecoder
 	{
 		// incomplete Huffman table. errors possible
-		HuffmanCode<uint8_t>{1,0b000000,0},
-		HuffmanCode<uint8_t>{2,0b000010,1},
-		HuffmanCode<uint8_t>{3,0b000110,2},
-		HuffmanCode<uint8_t>{4,0b001110,3},
-		HuffmanCode<uint8_t>{5,0b011110,4},
-		HuffmanCode<uint8_t>{6,0b111110,5}
+		HuffmanCode{1,0b000000,uint8_t{0}},
+		HuffmanCode{2,0b000010,uint8_t{1}},
+		HuffmanCode{3,0b000110,uint8_t{2}},
+		HuffmanCode{4,0b001110,uint8_t{3}},
+		HuffmanCode{5,0b011110,uint8_t{4}},
+		HuffmanCode{6,0b111110,uint8_t{5}}
 	};
 
 	HuffmanDecoder<int32_t> deltaDecoder
 	{
-		HuffmanCode<int32_t>{1,0b00,0},
-		HuffmanCode<int32_t>{2,0b10,1},
-		HuffmanCode<int32_t>{2,0b11,-1}
+		HuffmanCode{1,0b00,0},
+		HuffmanCode{2,0b10,1},
+		HuffmanCode{2,0b11,-1}
 	};
 
-	MemoryBuffer tmpBuffer(_blockSize);
-	uint8_t *tmpBufferPtr=tmpBuffer.data();
+	MemoryBuffer tmpBuffer{_blockSize};
 
 	// This is the dark, ancient secret of bzip2.
 	// versions before 0.9.5 had a data randomization for "too regular"
@@ -152,40 +148,40 @@ void BZIP2Decompressor::decompressImpl(Buffer &rawData,bool verify)
 
 	for (;;)
 	{
-		uint32_t blockHdrHigh=readBits(32);
-		uint32_t blockHdrLow=readBits(16);
+		uint32_t blockHdrHigh{readBits(32)};
+		uint32_t blockHdrLow{readBits(16)};
 		if (blockHdrHigh==0x31415926U && blockHdrLow==0x5359U)
 		{
 			// a block
 
 			// this is rather spaghetti...
 			readBits(32);	// block crc, not interested
-			bool randomized=readBit();
+			bool randomized{!!readBit()};
 
 			// basically the random inserted is one LSB after n-th bytes
 			// per defined in the table.
-			uint32_t randomPos=1;
-			uint32_t randomCounter=randomTable[0]-1;
+			uint32_t randomPos{1};
+			uint32_t randomCounter{randomTable[0]-1U};
 			auto randomBit=[&]()->bool
 			{
 				// Beauty is in the eye of the beholder: this is smallest form to hide the ugliness
 				return (!randomCounter--)?randomCounter=randomTable[randomPos++&511]:false;
 			};
 
-			uint32_t currentPtr=readBits(24);
+			uint32_t currentPtr{readBits(24)};
 
-			uint32_t currentBlockSize=0;
+			uint32_t currentBlockSize{0};
 			{
-				uint32_t numHuffmanItems=2;
-				uint32_t huffmanValues[256];
+				uint32_t numHuffmanItems{2};
+				std::array<uint32_t,256> huffmanValues;
 
 				{
 					// this is just a little bit inefficient but still we reading bit by bit since
 					// reference does it. (bitsream format details do not spill over)
-					std::vector<bool> usedMap(16);
+					std::array<bool,16> usedMap;
 					for (uint32_t i=0;i<16;i++) usedMap[i]=readBit();
 
-					std::vector<bool> huffmanMap(256);
+					std::array<bool,256> huffmanMap;
 					for (uint32_t i=0;i<16;i++)
 					{
 						for (uint32_t j=0;j<16;j++)
@@ -193,23 +189,26 @@ void BZIP2Decompressor::decompressImpl(Buffer &rawData,bool verify)
 					}
 
 					for (uint32_t i=0;i<256;i++) if (huffmanMap[i]) numHuffmanItems++;
-					if (numHuffmanItems==2) throw DecompressionError();
+					if (numHuffmanItems==2)
+						throw DecompressionError();
 
 					for (uint32_t currentValue=0,i=0;i<256;i++)
 						if (huffmanMap[i]) huffmanValues[currentValue++]=i;
 				}
 
-				uint32_t huffmanGroups=readBits(3);
-				if (huffmanGroups<2 || huffmanGroups>6) throw DecompressionError();
+				uint32_t huffmanGroups{readBits(3)};
+				if (huffmanGroups<2 || huffmanGroups>6)
+					throw DecompressionError();
 
-				uint32_t selectorsUsed=readBits(15);
-				if (!selectorsUsed) throw DecompressionError();
+				uint32_t selectorsUsed{readBits(15)};
+				if (!selectorsUsed)
+					throw DecompressionError();
 
-				MemoryBuffer huffmanSelectorList(selectorsUsed);
+				std::vector<uint8_t> huffmanSelectorList(selectorsUsed);
 
-				auto unMTF=[](uint8_t value,uint8_t map[])->uint8_t
+				auto unMTF=[](uint8_t value,auto &map)->uint8_t
 				{
-					uint8_t ret=map[value];
+					uint8_t ret{map[value]};
 					if (value)
 					{
 						uint8_t tmp=map[value];
@@ -221,24 +220,25 @@ void BZIP2Decompressor::decompressImpl(Buffer &rawData,bool verify)
 				};
 
 				// create Huffman selectors
-				uint8_t selectorMTFMap[6]={0,1,2,3,4,5};
+				std::array<uint8_t,6> selectorMTFMap{0,1,2,3,4,5};
 
 				for (uint32_t i=0;i<selectorsUsed;i++)
 				{
-					uint8_t item=unMTF(selectorDecoder.decode(readBit),selectorMTFMap);
-					if (item>=huffmanGroups) throw DecompressionError();
+					uint8_t item{unMTF(selectorDecoder.decode(readBit),selectorMTFMap)};
+					if (item>=huffmanGroups)
+						throw DecompressionError();
 					huffmanSelectorList[i]=item;
 				}
 
 				typedef HuffmanDecoder<uint32_t> BZIP2Decoder;
-				std::vector<BZIP2Decoder> dataDecoders(huffmanGroups);
+				std::vector<BZIP2Decoder> dataDecoders{huffmanGroups};
 
 				// Create all tables
 				for (uint32_t i=0;i<huffmanGroups;i++)
 				{
-					uint8_t bitLengths[258];
+					std::array<uint8_t,258> bitLengths;
 
-					uint32_t currentBits=readBits(5);
+					uint32_t currentBits{readBits(5)};
 					for (uint32_t j=0;j<numHuffmanItems;j++)
 					{
 						int32_t delta;
@@ -247,7 +247,8 @@ void BZIP2Decompressor::decompressImpl(Buffer &rawData,bool verify)
 							delta=deltaDecoder.decode(readBit);
 							currentBits+=delta;
 						} while (delta);
-						if (currentBits<1 || currentBits>20) throw DecompressionError();
+						if (currentBits<1 || currentBits>20)
+							throw DecompressionError();
 						bitLengths[j]=currentBits;
 					}
 
@@ -255,20 +256,21 @@ void BZIP2Decompressor::decompressImpl(Buffer &rawData,bool verify)
 				}
 
 				// Huffman decode + unRLE + unMTF
-				BZIP2Decoder *currentHuffmanDecoder=nullptr;
-				uint32_t currentHuffmanIndex=0;
-				uint8_t dataMTFMap[256];
+				BZIP2Decoder *currentHuffmanDecoder{nullptr};
+				uint32_t currentHuffmanIndex{0};
+				std::array<uint8_t,256> dataMTFMap;
 				for (uint32_t i=0;i<numHuffmanItems-2;i++) dataMTFMap[i]=i;
 
-				uint32_t currentRunLength=0;
-				uint32_t currentRLEWeight=1;
+				uint32_t currentRunLength{0};
+				uint32_t currentRLEWeight{1};
 
 				auto decodeRLE=[&]()
 				{
 					if (currentRunLength)
 					{
-						if (currentBlockSize+currentRunLength>_blockSize) throw DecompressionError();
-						for (uint32_t i=0;i<currentRunLength;i++) tmpBufferPtr[currentBlockSize++]=huffmanValues[dataMTFMap[0]];
+						if (currentBlockSize+currentRunLength>_blockSize)
+							throw DecompressionError();
+						for (uint32_t i=0;i<currentRunLength;i++) tmpBuffer[currentBlockSize++]=huffmanValues[dataMTFMap[0]];
 					}
 					currentRunLength=0;
 					currentRLEWeight=1;
@@ -278,27 +280,31 @@ void BZIP2Decompressor::decompressImpl(Buffer &rawData,bool verify)
 				{
 					if (!(streamIndex%50))
 					{
-						if (currentHuffmanIndex>=selectorsUsed) throw DecompressionError();
+						if (currentHuffmanIndex>=selectorsUsed)
+							throw DecompressionError();
 						currentHuffmanDecoder=&dataDecoders[huffmanSelectorList[currentHuffmanIndex++]];
 					}
-					uint32_t symbolMTF=currentHuffmanDecoder->decode(readBit);
+					uint32_t symbolMTF{currentHuffmanDecoder->decode(readBit)};
 					// stop marker is referenced only once, and it is the last one
 					// This means we do no have to un-MTF it for detection
 					if (symbolMTF==numHuffmanItems-1) break;
-					if (currentBlockSize>=_blockSize) throw DecompressionError();
+					if (currentBlockSize>=_blockSize)
+						throw DecompressionError();
 					if (symbolMTF<2)
 					{
 						currentRunLength+=currentRLEWeight<<symbolMTF;
 						currentRLEWeight<<=1;
 					} else {
 						decodeRLE();
-						uint8_t symbol=unMTF(symbolMTF-1,dataMTFMap);
-						if (currentBlockSize>=_blockSize) throw DecompressionError();
-						tmpBufferPtr[currentBlockSize++]=huffmanValues[symbol];
+						uint8_t symbol{unMTF(symbolMTF-1,dataMTFMap)};
+						if (currentBlockSize>=_blockSize)
+							throw DecompressionError();
+						tmpBuffer[currentBlockSize++]=huffmanValues[symbol];
 					}
 				}
 				decodeRLE();
-				if (currentPtr>=currentBlockSize) throw DecompressionError();
+				if (currentPtr>=currentBlockSize)
+					throw DecompressionError();
 			}
 
 			// inverse BWT + final RLE decoding.
@@ -306,15 +312,13 @@ void BZIP2Decompressor::decompressImpl(Buffer &rawData,bool verify)
 			// 1. Can the stream end at 4 literals without count? I assume it is a valid optimization (and that this does not spillover to next block)
 			// 2. Can the RLE-step include counts 252 to 255 even if reference does not do them? I assume yes here as here as well
 			// 3. Can the stream be empty? We do not take issue here about that (that should be culled out earlier already)
-			uint32_t sums[256];
+			std::array<uint32_t,256> sums;
 			for (uint32_t i=0;i<256;i++) sums[i]=0;
 
 			for (uint32_t i=0;i<currentBlockSize;i++)
-			{
-				sums[tmpBufferPtr[i]]++;
-			}
+				sums[tmpBuffer[i]]++;
 
-			uint32_t rank[256];
+			std::array<uint32_t,256> rank;
 			for (uint32_t tot=0,i=0;i<256;i++)
 			{
 				rank[i]=tot;
@@ -326,14 +330,13 @@ void BZIP2Decompressor::decompressImpl(Buffer &rawData,bool verify)
 			// since by calculating forward table we can do forward decoding of the data on the same pass as iBWT
 			//
 			// also, because I'm lazy
-			MemoryBuffer forwardIndex(currentBlockSize*sizeof(uint32_t));
-			auto forwardIndexPtr=forwardIndex.cast<uint32_t>();
+			std::vector<uint32_t> forwardIndex(currentBlockSize);
 			for (uint32_t i=0;i<currentBlockSize;i++)
-				forwardIndexPtr[rank[tmpBufferPtr[i]]++]=i;
+				forwardIndex[rank[tmpBuffer[i]]++]=i;
 
 			// output + final RLE decoding
-			uint8_t currentCh=0;
-			uint32_t currentChCount=0;
+			uint8_t currentCh{0};
+			uint32_t currentChCount{0};
 			auto outputByte=[&](uint8_t ch)
 			{
 				if (randomized && randomBit()) ch^=1;
@@ -364,13 +367,13 @@ void BZIP2Decompressor::decompressImpl(Buffer &rawData,bool verify)
 				}
 			};
 
-			size_t destOffsetStart=outputStream.getOffset();
+			size_t destOffsetStart{outputStream.getOffset()};
 
 			// and now the final iBWT + unRLE is easy...
 			for (uint32_t i=0;i<currentBlockSize;i++)
 			{
-				currentPtr=forwardIndexPtr[currentPtr];
-				outputByte(tmpBufferPtr[currentPtr]);
+				currentPtr=forwardIndex[currentPtr];
+				outputByte(tmpBuffer[currentPtr]);
 			}
 			// cleanup the state, a bit hackish way to do it
 			if (currentChCount) outputByte(currentChCount==4?0:~currentCh);
@@ -380,15 +383,15 @@ void BZIP2Decompressor::decompressImpl(Buffer &rawData,bool verify)
 
 		} else if (blockHdrHigh==0x17724538U && blockHdrLow==0x5090U) {
 			// end of blocks
-			uint32_t rawCRC=readBits(32);
-			if (verify && crc!=rawCRC) throw VerificationError();
+			uint32_t rawCRC{readBits(32)};
+			if (verify && crc!=rawCRC)
+				throw VerificationError();
 			break;
 		} else throw DecompressionError();
 	}
 
-	if (!_rawSize) _rawSize=outputStream.getOffset();
-	if (!_packedSize) _packedSize=inputStream.getOffset();
-	if (_rawSize!=outputStream.getOffset()) throw DecompressionError();
+	_rawSize=outputStream.getOffset();
+	_packedSize=inputStream.getOffset();
 }
 
 void BZIP2Decompressor::decompressImpl(Buffer &rawData,const Buffer &previousData,bool verify)
