@@ -15,16 +15,16 @@ public:
 	DEFLATEDecompressor(const Buffer &packedData,bool exactSizeKnown,bool verify);
 	DEFLATEDecompressor(uint32_t hdr,uint32_t recursionLevel,const Buffer &packedData,std::shared_ptr<XPKDecompressor::State> &state,bool verify);
 	DEFLATEDecompressor(const Buffer &packedData,size_t packedSize,size_t rawSize,bool isZlib,bool verify,bool deflate64);		// zlib or completely raw stream
-	virtual ~DEFLATEDecompressor();
+	~DEFLATEDecompressor() noexcept=default;
 
-	virtual size_t getRawSize() const noexcept override final;
-	virtual size_t getPackedSize() const noexcept override final;
+	size_t getRawSize() const noexcept final;
+	size_t getPackedSize() const noexcept final;
 
-	virtual const std::string &getName() const noexcept override final;
-	virtual const std::string &getSubName() const noexcept override final;
+	const std::string &getName() const noexcept final;
+	const std::string &getSubName() const noexcept final;
 
-	virtual void decompressImpl(Buffer &rawData,bool verify) override final;
-	virtual void decompressImpl(Buffer &rawData,const Buffer &previousData,bool verify) override final;
+	void decompressImpl(Buffer &rawData,bool verify) final;
+	void decompressImpl(Buffer &rawData,const Buffer &previousData,bool verify) final;
 
 	static bool detectHeader(uint32_t hdr) noexcept;
 	static bool detectHeaderXPK(uint32_t hdr) noexcept;
@@ -38,18 +38,19 @@ private:
 	enum class Type
 	{
 		GZIP=0,
+		Quasijarus,
 		ZLib,
 		Raw
 	};
 
 	const Buffer	&_packedData;
 
-	size_t		_packedSize=0;
-	size_t		_packedOffset=0;
-	size_t		_rawSize=0;
+	size_t		_packedSize{0};
+	size_t		_packedOffset{0};
+	size_t		_rawSize{0};
 	Type		_type;
-	bool		_exactSizeKnown=true;
-	bool		_deflate64=false;
+	bool		_exactSizeKnown{true};
+	bool		_deflate64{false};
 };
 
 }
