@@ -9,7 +9,7 @@
 #include "DynamicHuffmanDecoder.hpp"
 #include "InputStream.hpp"
 #include "OutputStream.hpp"
-
+#include "VariableLengthCodeDecoder.hpp"
 #include "common/MemoryBuffer.hpp"
 #include "common/CRC16.hpp"
 #include "common/OverflowCheck.hpp"
@@ -30,40 +30,46 @@ std::shared_ptr<Decompressor> DMSDecompressor::create(const Buffer &packedData,b
 }
 
 DMSDecompressor::DMSDecompressor(const Buffer &packedData,bool verify) :
-	_packedData(packedData)
+	_packedData{packedData}
 {
-	uint32_t hdr=packedData.readBE32(0);
-	if (!detectHeader(hdr) || packedData.size()<56) throw InvalidFormatError();
+	uint32_t hdr{packedData.readBE32(0)};
+	if (!detectHeader(hdr) || packedData.size()<56)
+		throw InvalidFormatError();
 
 	if (verify && CRC16(packedData,4,50,0)!=packedData.readBE16(54))
 		throw VerificationError();
 
-	uint16_t info=packedData.readBE16(10);
+	uint16_t info{packedData.readBE16(10)};
 	_isObsfuscated=info&2;				// using 16 bit key is not encryption, it is obsfuscation
 	_isHD=info&16;
-	if (info&32) throw InvalidFormatError();	// MS-DOS disk
+	if (info&32)
+		throw InvalidFormatError();		// MS-DOS disk
 
 	// packed data in header is useless, as is rawsize and track numbers
 	// they are not always filled
 
-	if (packedData.readBE16(50)>6) throw InvalidFormatError(); // either FMS or unknown
+	if (packedData.readBE16(50)>6)
+		throw InvalidFormatError();		// either FMS or unknown
+
+	const std::array<uint32_t,7> contextSizes{0,0,256,16384,16384,4096,8192};
 
 	// now calculate the real packed size, including headers
-	uint32_t offset=56;
-	uint32_t accountedSize=0;
-	uint32_t lastTrackSize=0;
-	uint32_t numTracks=0;
-	uint32_t minTrack=80;
-	uint32_t prevTrack=0;
+	uint32_t offset{56};
+	uint32_t accountedSize{0};
+	uint32_t lastTrackSize{0};
+	uint32_t numTracks{0};
+	uint32_t minTrack{80};
+	uint32_t prevTrack{0};
 	while (offset+20<packedData.size())
 	{
 		if (_packedData.readBE16(offset)!=MultiChar2("TR"))
 		{
 			// secondary exit criteria, should not be like this, if the header would be trustworthy
-			if (!accountedSize) throw InvalidFormatError();
+			if (!accountedSize)
+				throw InvalidFormatError();
 			break;
 		}
-		uint32_t trackNo=_packedData.readBE16(offset+2);
+		uint32_t trackNo{_packedData.readBE16(offset+2)};
 		// lets not go backwards on tracks!
 		if (trackNo<prevTrack) break;
 
@@ -71,17 +77,16 @@ DMSDecompressor::DMSDecompressor(const Buffer &packedData,bool verify) :
 		if (verify && CRC16(packedData,offset,18,0)!=packedData.readBE16(offset+18))
 			throw VerificationError();
 
-		uint8_t mode=_packedData.read8(offset+13);
-		if (mode>6) throw InvalidFormatError();
-		static const uint32_t contextSizes[7]={0,0,256,16384,16384,4096,8192};
+		uint8_t mode{_packedData.read8(offset+13)};
+		if (mode>6)
+			throw InvalidFormatError();
+
 		_contextBufferSize=std::max(_contextBufferSize,contextSizes[mode]);
 
-		uint8_t flags=_packedData.read8(offset+12);
+		uint8_t flags{_packedData.read8(offset+12)};
 		if ((mode>=2 && mode<=4) || (mode>=5 && (flags&4)))
-		{
 			_tmpBufferSize=std::max(_tmpBufferSize,uint32_t(_packedData.readBE16(offset+8)));
-		}
-		uint32_t packedChunkLength=packedData.readBE16(offset+6);
+		uint32_t packedChunkLength{packedData.readBE16(offset+6)};
 		if (OverflowCheck::sum(offset,20U,packedChunkLength)>packedData.size())
 			throw InvalidFormatError();
 		if (verify && CRC16(packedData,offset+20,packedChunkLength,0)!=packedData.readBE16(offset+16))
@@ -100,7 +105,7 @@ DMSDecompressor::DMSDecompressor(const Buffer &packedData,bool verify) :
 		// this is the real exit critea, unfortunately
 		if (trackNo>=79 && trackNo<0x8000U) break;
 	}
-	uint32_t trackSize=(_isHD)?22528:11264;
+	uint32_t trackSize{(_isHD)?22528U:11264U};
 	_rawOffset=minTrack*trackSize;
 	if (minTrack>=numTracks)
 		throw InvalidFormatError();
@@ -113,14 +118,9 @@ DMSDecompressor::DMSDecompressor(const Buffer &packedData,bool verify) :
 		throw InvalidFormatError();
 }
 
-DMSDecompressor::~DMSDecompressor()
-{
-	// nothing needed
-}
-
 const std::string &DMSDecompressor::getName() const noexcept
 {
-	static std::string name="DMS: Disk Masher System";
+	static std::string name{"DMS: Disk Masher System"};
 	return name;
 }
 
@@ -146,7 +146,7 @@ size_t DMSDecompressor::getImageOffset() const noexcept
 
 void DMSDecompressor::decompressImpl(Buffer &rawData,bool verify)
 {
-	uint32_t restartPosition=0;
+	uint32_t restartPosition{0};
 	if (!_isObsfuscated)
 	{
 		decompressImpl(rawData,verify,restartPosition);
@@ -174,20 +174,22 @@ void DMSDecompressor::decompressImpl(Buffer &rawData,bool verify)
 // needs to be split
 void DMSDecompressor::decompressImpl(Buffer &rawData,bool verify,uint32_t &restartPosition)
 {
-	if (rawData.size()<_rawSize) throw DecompressionError();
-	MemoryBuffer contextBuffer(_contextBufferSize);
-	MemoryBuffer tmpBuffer(_tmpBufferSize);
-	uint32_t limitedDecompress=~0U;
+	if (rawData.size()<_rawSize)
+		throw DecompressionError();
+	MemoryBuffer contextBuffer{_contextBufferSize};
+	MemoryBuffer tmpBuffer{_tmpBufferSize};
+	uint32_t limitedDecompress{~0U};
 
 	class ObsfuscatedInputStream
 	{
 	public:
 		ObsfuscatedInputStream(const Buffer &data,uint32_t start,uint32_t length,bool obsfuscate) :
-			_inputStream(data,start,OverflowCheck::sum(start,length)),
-			_obsfuscate(obsfuscate)
+			_inputStream{data,start,OverflowCheck::sum(start,length)},
+			_obsfuscate{obsfuscate}
 		{
 			// nothing needed
 		}
+		~ObsfuscatedInputStream() noexcept=default;
 
 		void reset(size_t startOffset,size_t endOffset)
 		{
@@ -196,28 +198,32 @@ void DMSDecompressor::decompressImpl(Buffer &rawData,bool verify,uint32_t &resta
 
 		uint8_t readByte()
 		{
-			uint8_t ch=_inputStream.readByte();
+			uint8_t ch{_inputStream.readByte()};
 			if (!_obsfuscate)
 			{
 				return ch;
 			} else {
-				uint8_t ret=ch^_passAccumulator;
+				uint8_t ret{uint8_t(ch^_passAccumulator)};
 				_passAccumulator=(_passAccumulator>>1)+uint16_t(ch);
 				return ret;
 			}
 		}
 
-		void setCode(uint16_t passAccumulator) { _passAccumulator=passAccumulator; }
+		void setCode(uint16_t passAccumulator)
+		{
+			_passAccumulator=passAccumulator;
+		}
+
 		bool eof() const { return _inputStream.getOffset()==_inputStream.getEndOffset(); }
 
 	private:
 		ForwardInputStream	_inputStream;
 		bool			_obsfuscate;
-		uint16_t		_passAccumulator=0;
+		uint16_t		_passAccumulator{0};
 	};
 
-	ObsfuscatedInputStream inputStream(_packedData,0,0,_isObsfuscated);
-	MSBBitReader<ObsfuscatedInputStream> bitReader(inputStream);
+	ObsfuscatedInputStream inputStream{_packedData,0,0,_isObsfuscated};
+	MSBBitReader<ObsfuscatedInputStream> bitReader{inputStream};
 	auto initInputStream=[&](uint32_t start,uint32_t length)
 	{
 		inputStream.reset(start,OverflowCheck::sum(start,length));
@@ -238,7 +244,7 @@ void DMSDecompressor::decompressImpl(Buffer &rawData,bool verify,uint32_t &resta
 		return bitReader.readBits8(1);
 	};
 
-	ForwardOutputStream outputStream(rawData,0,0);
+	ForwardOutputStream outputStream{rawData,0,0};
 	auto initOutputStream=[&](uint32_t start,uint32_t length)
 	{
 		outputStream.reset(start,OverflowCheck::sum(start,length));
@@ -247,7 +253,7 @@ void DMSDecompressor::decompressImpl(Buffer &rawData,bool verify,uint32_t &resta
 	// fill unused tracks with zeros
 	std::memset(rawData.data(),0,_rawSize);
 
-	bool doInitContext=true;
+	bool doInitContext{true};
 	// quick: context used is 256 bytes
 	// medium & deep: context used is 16384 bytes
 	// heavy: context used is 4096/8192 bytes
@@ -264,10 +270,10 @@ void DMSDecompressor::decompressImpl(Buffer &rawData,bool verify,uint32_t &resta
 		}
 	};
 
-	auto checksum=[](const uint8_t *src,uint32_t srcLength)->uint16_t
+	auto checksum=[](const Buffer &src,uint32_t offset,uint32_t srcLength)->uint16_t
 	{
-		uint16_t ret=0;
-		for (uint32_t i=0;i<srcLength;i++) ret+=uint16_t(src[i]);
+		uint16_t ret{0};
+		for (uint32_t i=0;i<srcLength;i++) ret+=uint16_t(src[i+offset]);
 		return ret;
 	};
 
@@ -283,8 +289,8 @@ void DMSDecompressor::decompressImpl(Buffer &rawData,bool verify,uint32_t &resta
 		while (!output.eof())
 		{
 			if (output.getOffset()>=limitedDecompress) return;
-			uint8_t ch=input.readByte();
-			uint32_t count=1;
+			uint8_t ch{input.readByte()};
+			uint32_t count{1};
 			if (ch==0x90U)
 			{
 				count=input.readByte();
@@ -312,8 +318,8 @@ void DMSDecompressor::decompressImpl(Buffer &rawData,bool verify,uint32_t &resta
 				output.writeByte(contextBuffer[contextLocation++]=readBits(8));
 				contextLocation&=0xffU;
 			} else {
-				uint32_t count=readBits(2)+2;
-				uint8_t offset=contextLocation-readBits(8)-1;
+				uint32_t count{readBits(2)+2};
+				uint8_t offset{uint8_t(contextLocation-readBits(8)-1)};
 				for (uint32_t i=0;i<count;i++)
 				{
 					output.writeByte(contextBuffer[contextLocation++]=contextBuffer[(i+offset)&0xffU]);
@@ -325,52 +331,8 @@ void DMSDecompressor::decompressImpl(Buffer &rawData,bool verify,uint32_t &resta
 		contextLocation&=0xffU;
 	};
 
-	static const uint8_t lengthTable[256]={
-		 0, 0, 0, 0, 0, 0, 0, 0,  0, 0, 0, 0, 0, 0, 0, 0,
-		 0, 0, 0, 0, 0, 0, 0, 0,  0, 0, 0, 0, 0, 0, 0, 0,
-		 1, 1, 1, 1, 1, 1, 1, 1,  1, 1, 1, 1, 1, 1, 1, 1,
-		 2, 2, 2, 2, 2, 2, 2, 2,  2, 2, 2, 2, 2, 2, 2, 2,
-		 3, 3, 3, 3, 3, 3, 3, 3,  3, 3, 3, 3, 3, 3, 3, 3,
-		 4, 4, 4, 4, 4, 4, 4, 4,  5, 5, 5, 5, 5, 5, 5, 5,
-		 6, 6, 6, 6, 6, 6, 6, 6,  7, 7, 7, 7, 7, 7, 7, 7,
-		 8, 8, 8, 8, 8, 8, 8, 8,  9, 9, 9, 9, 9, 9, 9, 9,
-		10,10,10,10,10,10,10,10, 11,11,11,11,11,11,11,11,
-		12,12,12,12,13,13,13,13, 14,14,14,14,15,15,15,15,
-		16,16,16,16,17,17,17,17, 18,18,18,18,19,19,19,19,
-		20,20,20,20,21,21,21,21, 22,22,22,22,23,23,23,23,
-		24,24,25,25,26,26,27,27, 28,28,29,29,30,30,31,31,
-		32,32,33,33,34,34,35,35, 36,36,37,37,38,38,39,39,
-		40,40,41,41,42,42,43,43, 44,44,45,45,46,46,47,47,
-		48,49,50,51,52,53,54,55, 56,57,58,59,60,61,62,63};
 
-	static const uint8_t bitLengthTable[256]={
-		3,3,3,3,3,3,3,3, 3,3,3,3,3,3,3,3,
-		3,3,3,3,3,3,3,3, 3,3,3,3,3,3,3,3,
-		4,4,4,4,4,4,4,4, 4,4,4,4,4,4,4,4,
-		4,4,4,4,4,4,4,4, 4,4,4,4,4,4,4,4,
-		4,4,4,4,4,4,4,4, 4,4,4,4,4,4,4,4,
-		5,5,5,5,5,5,5,5, 5,5,5,5,5,5,5,5,
-		5,5,5,5,5,5,5,5, 5,5,5,5,5,5,5,5,
-		5,5,5,5,5,5,5,5, 5,5,5,5,5,5,5,5,
-		5,5,5,5,5,5,5,5, 5,5,5,5,5,5,5,5,
-		6,6,6,6,6,6,6,6, 6,6,6,6,6,6,6,6,
-		6,6,6,6,6,6,6,6, 6,6,6,6,6,6,6,6,
-		6,6,6,6,6,6,6,6, 6,6,6,6,6,6,6,6,
-		7,7,7,7,7,7,7,7, 7,7,7,7,7,7,7,7,
-		7,7,7,7,7,7,7,7, 7,7,7,7,7,7,7,7,
-		7,7,7,7,7,7,7,7, 7,7,7,7,7,7,7,7,
-		8,8,8,8,8,8,8,8, 8,8,8,8,8,8,8,8};
-
-	auto decodeLengthValueHalf=[&](uint8_t code)->uint32_t
-	{
-		return (((code<<bitLengthTable[code])|readBits(bitLengthTable[code]))&0xffU);
-	};
-
-	auto decodeLengthValueFull=[&](uint8_t code)->uint32_t
-	{
-		return (uint32_t(lengthTable[code])<<8)|
-			uint32_t(((code<<bitLengthTable[code])|readBits(bitLengthTable[code]))&0xffU);
-	};
+	VariableLengthCodeDecoder lengthDecoder{7,7,8,8,8,9,9,9,9,10,10,10,11,11,11,12};
 
 	auto unpackMedium=[&](auto &output)
 	{
@@ -384,12 +346,14 @@ void DMSDecompressor::decompressImpl(Buffer &rawData,bool verify,uint32_t &resta
 				output.writeByte(contextBuffer[contextLocation++]=readBits(8));
 				contextLocation&=0x3fffU;
 			} else {
-				uint32_t code=readBits(8);
-				uint32_t count=lengthTable[code]+3;
+				uint32_t count{lengthDecoder.decode(readBits,readBits(4U))};
+				uint32_t rawDist{lengthDecoder.decode([&](uint32_t c)
+				{
+					return (uint32_t(count&0xfU)<<(c-4U))|readBits(c-4U);
+				},((count>>4U)&0xfU))};
+				count=(count>>8U)+3U;
 
-				uint32_t tmp=decodeLengthValueFull(decodeLengthValueHalf(code));
-
-				uint32_t offset=contextLocation-tmp-1;
+				uint32_t offset{contextLocation-rawDist-1U};
 				for (uint32_t i=0;i<count;i++)
 				{
 					output.writeByte(contextBuffer[contextLocation++]=contextBuffer[(i+offset)&0x3fffU]);
@@ -417,8 +381,8 @@ void DMSDecompressor::decompressImpl(Buffer &rawData,bool verify,uint32_t &resta
 				output.writeByte(contextBuffer[contextLocation++]=symbol);
 				contextLocation&=0x3fffU;
 			} else {
-				uint32_t count=symbol-253;	// minimum repeat is 3
-				uint32_t offset=contextLocation-decodeLengthValueFull(readBits(8))-1;
+				uint32_t count{symbol-253};	// minimum repeat is 3
+				uint32_t offset{contextLocation-lengthDecoder.decode(readBits,readBits(4U))-1U};
 
 				for (uint32_t i=0;i<count;i++)
 				{
@@ -433,7 +397,7 @@ void DMSDecompressor::decompressImpl(Buffer &rawData,bool verify,uint32_t &resta
 
 	// these are not part of the initContext like other methods
 	std::unique_ptr<OptionalHuffmanDecoder<uint32_t>> symbolDecoder,offsetDecoder;
-	bool heavyLastInitialized=false;		// this is part of initContext on some implementations. screwy!!!
+	bool heavyLastInitialized{false};		// this is part of initContext on some implementations. screwy!!!
 	uint32_t heavyLastOffset;
 	auto unpackHeavy=[&](auto &output,bool initTables,bool use8kDict)
 	{
@@ -448,16 +412,16 @@ void DMSDecompressor::decompressImpl(Buffer &rawData,bool verify,uint32_t &resta
 		auto readTable=[&](std::unique_ptr<OptionalHuffmanDecoder<uint32_t>> &decoder,uint32_t countBits,uint32_t valueBits)
 		{
 			decoder=std::make_unique<OptionalHuffmanDecoder<uint32_t>>();
-			uint32_t count=readBits(countBits);
+			uint32_t count{readBits(countBits)};
 			if (count)
 			{
-				uint8_t lengthBuffer[512];
+				std::array<uint8_t,512> lengthBuffer;
 				// in order to speed up the deObsfuscation, do not send the hopeless
 				// data into slow CreateOrderlyHuffmanTable
-				uint64_t sum=0;
+				uint64_t sum{0};
 				for (uint32_t i=0;i<count;i++)
 				{
-					uint32_t bits=readBits(valueBits);
+					uint32_t bits{readBits(valueBits)};
 					if (bits)
 					{
 						sum+=uint64_t(1U)<<(32-bits);
@@ -468,7 +432,7 @@ void DMSDecompressor::decompressImpl(Buffer &rawData,bool verify,uint32_t &resta
 				}
 				decoder->createOrderlyHuffmanTable(lengthBuffer,count);
 			} else {
-				uint32_t index=readBits(countBits);
+				uint32_t index{readBits(countBits)};
 				decoder->setEmpty(index);
 			}
 		};
@@ -479,8 +443,8 @@ void DMSDecompressor::decompressImpl(Buffer &rawData,bool verify,uint32_t &resta
 			readTable(offsetDecoder,5,4);
 		}
 
-		uint32_t mask=use8kDict?0x1fffU:0xfffU;
-		uint32_t bitLength=use8kDict?14:13;
+		uint32_t mask{use8kDict?0x1fffU:0xfffU};
+		uint32_t bitLength{use8kDict?14U:13U};
 
 		while (!output.eof())
 		{
@@ -491,16 +455,16 @@ void DMSDecompressor::decompressImpl(Buffer &rawData,bool verify,uint32_t &resta
 				output.writeByte(contextBuffer[contextLocation++]=symbol);
 				contextLocation&=mask;
 			} else {
-				uint32_t count=symbol-253;	// minimum repeat is 3
-				uint32_t offsetLength=offsetDecoder->decode(readBit);
-				uint32_t rawOffset=heavyLastOffset;
+				uint32_t count{symbol-253};	// minimum repeat is 3
+				uint32_t offsetLength{offsetDecoder->decode(readBit)};
+				uint32_t rawOffset{heavyLastOffset};
 				if (offsetLength!=bitLength)
 				{
 					if (offsetLength) rawOffset=(1<<(offsetLength-1))|readBits(offsetLength-1);
 						else rawOffset=0;
 					heavyLastOffset=rawOffset;
 				}
-				uint32_t offset=contextLocation-rawOffset-1;
+				uint32_t offset{contextLocation-rawOffset-1};
 				for (uint32_t i=0;i<count;i++)
 				{
 					output.writeByte(contextBuffer[contextLocation++]=contextBuffer[(i+offset)&mask]);
@@ -510,19 +474,19 @@ void DMSDecompressor::decompressImpl(Buffer &rawData,bool verify,uint32_t &resta
 		}
 	};
 
-	bool _codeFixed=false;
+	bool _codeFixed{false};
 	uint32_t trackLength=(_isHD)?22528:11264;
 	for (uint32_t packedOffset=56,packedChunkLength=0;packedOffset!=_packedSize;packedOffset=OverflowCheck::sum(packedOffset,20U,packedChunkLength))
 	{
 		// There are some info tracks, at -1 or 80. ignore those (if still present)
-		uint16_t trackNo=_packedData.readBE16(packedOffset+2);
+		uint16_t trackNo{_packedData.readBE16(packedOffset+2)};
 		packedChunkLength=_packedData.readBE16(packedOffset+6);
 		if (trackNo==80) break;							// should not happen, this is already excluded
 		// even though only -1 should be used I've seen -2 as well. ignore all negatives
-		uint32_t tmpChunkLength=_packedData.readBE16(packedOffset+8);		// after the first unpack (if twostage)
-		uint32_t rawChunkLength=_packedData.readBE16(packedOffset+10);		// after final unpack
-		uint8_t flags=_packedData.read8(packedOffset+12);
-		uint8_t mode=_packedData.read8(packedOffset+13);
+		uint32_t tmpChunkLength{_packedData.readBE16(packedOffset+8)};		// after the first unpack (if twostage)
+		uint32_t rawChunkLength{_packedData.readBE16(packedOffset+10)};		// after final unpack
+		uint8_t flags{_packedData.read8(packedOffset+12)};
+		uint8_t mode{_packedData.read8(packedOffset+13)};
 
 		// could affect context, but in practice they are separate, even though there is no explicit reset
 		// deal with decompression though...
@@ -532,11 +496,14 @@ void DMSDecompressor::decompressImpl(Buffer &rawData,bool verify,uint32_t &resta
 			finishStream();
 			continue;
 		}
-		if (rawChunkLength>trackLength) throw DecompressionError();
-		if (trackNo>80) throw DecompressionError();				// should not happen, already excluded
+		if (rawChunkLength>trackLength)
+			throw DecompressionError();
+		if (trackNo>80)
+			throw DecompressionError();					// should not happen, already excluded
 
-		uint32_t dataOffset=trackNo*trackLength;
-		if (_rawOffset>dataOffset) throw DecompressionError();
+		uint32_t dataOffset{trackNo*trackLength};
+		if (_rawOffset>dataOffset)
+			throw DecompressionError();
 
 		auto handleTrackSize=[&]()
 		{
@@ -550,11 +517,11 @@ void DMSDecompressor::decompressImpl(Buffer &rawData,bool verify,uint32_t &resta
 		{
 			auto applyFix=[&]()
 			{
-				if (mode==6U && !_isObsfuscated)
+				if (mode>=5U && !_isObsfuscated)
 				{
-					uint32_t missingNo=uint32_t(outputStream.getEndOffset()-outputStream.getOffset());
-					uint16_t protoSum=checksum(&rawData[dataOffset-_rawOffset],rawChunkLength-missingNo);
-					uint16_t fileSum=_packedData.readBE16(packedOffset+14);
+					uint32_t missingNo{uint32_t(outputStream.getEndOffset()-outputStream.getOffset())};
+					uint16_t protoSum{checksum(rawData,dataOffset-_rawOffset,rawChunkLength-missingNo)};
+					uint16_t fileSum{_packedData.readBE16(packedOffset+14)};
 
 					// too many ways to interpret the data
 					if (missingNo>1)
@@ -565,7 +532,7 @@ void DMSDecompressor::decompressImpl(Buffer &rawData,bool verify,uint32_t &resta
 					{
 						// last byte can be trashed by the compressor, but fortunately it can be fixed
 						protoSum-=*outputStream.history(1);
-						uint16_t fixByte=fileSum-protoSum;
+						uint16_t fixByte{uint16_t(fileSum-protoSum)};
 						if (fixByte>=0x100U)
 							throw DecompressionError();
 						*outputStream.history(1)=uint8_t(fixByte);
@@ -576,7 +543,7 @@ void DMSDecompressor::decompressImpl(Buffer &rawData,bool verify,uint32_t &resta
 			initInputStream(packedOffset+20,packedChunkLength);
 			if (doRLE)
 			{
-				ForwardOutputStream tmpOutputStream(tmpBuffer,0,tmpChunkLength);
+				ForwardOutputStream tmpOutputStream{tmpBuffer,0,tmpChunkLength};
 				try
 				{
 					func(tmpOutputStream,params...);
@@ -584,8 +551,8 @@ void DMSDecompressor::decompressImpl(Buffer &rawData,bool verify,uint32_t &resta
 					// nothing needed
 				}
 				finishStream();
-				uint32_t missingNo=static_cast<uint32_t>(tmpOutputStream.getEndOffset()-tmpOutputStream.getOffset());
-				ForwardInputStream tmpInputStream(tmpBuffer,0,tmpChunkLength-missingNo);
+				uint32_t missingNo{static_cast<uint32_t>(tmpOutputStream.getEndOffset()-tmpOutputStream.getOffset())};
+				ForwardInputStream tmpInputStream{tmpBuffer,0,tmpChunkLength-missingNo};
 				initOutputStream(dataOffset-_rawOffset,rawChunkLength);
 				try
 				{
@@ -627,7 +594,7 @@ void DMSDecompressor::decompressImpl(Buffer &rawData,bool verify,uint32_t &resta
 					inputStream.setCode(restartPosition);
 					limitedDecompress=~0U;
 					processBlock(doRLE,func,params...);
-					if (checksum(&rawData[dataOffset-_rawOffset],rawChunkLength)!=_packedData.readBE16(packedOffset+14)) continue;
+					if (checksum(rawData,dataOffset-_rawOffset,rawChunkLength)!=_packedData.readBE16(packedOffset+14)) continue;
 					_codeFixed=true;
 					return;
 				} catch (const Buffer::Error &) {
@@ -646,7 +613,7 @@ void DMSDecompressor::decompressImpl(Buffer &rawData,bool verify,uint32_t &resta
 					doInitContext=true;
 					inputStream.setCode(restartPosition);
 					processBlock(doRLE,func,params...);
-					if (checksum(&rawData[dataOffset-_rawOffset],rawChunkLength)!=_packedData.readBE16(packedOffset+14)) continue;
+					if (checksum(rawData,dataOffset-_rawOffset,rawChunkLength)!=_packedData.readBE16(packedOffset+14)) continue;
 					_codeFixed=true;
 					return;
 				} catch (const Buffer::Error &) {
@@ -690,6 +657,7 @@ void DMSDecompressor::decompressImpl(Buffer &rawData,bool verify,uint32_t &resta
 			// 4: do RLE
 			// heavy1 uses 4k dictionary (mode 5), whereas heavy2 uses 8k dictionary
 			case 5:
+			[[fallthrough]];
 			case 6:
 			processBlockCode(flags&4,unpackHeavy,flags&2,mode==6);
 			break;
@@ -699,7 +667,7 @@ void DMSDecompressor::decompressImpl(Buffer &rawData,bool verify,uint32_t &resta
 		}
 		if (!(flags&1)) doInitContext=true;
 
-		if (verify && checksum(&rawData[dataOffset-_rawOffset],rawChunkLength)!=_packedData.readBE16(packedOffset+14))
+		if (verify && checksum(rawData,dataOffset-_rawOffset,rawChunkLength)!=_packedData.readBE16(packedOffset+14))
 			throw VerificationError();
 	}
 }
