@@ -5,8 +5,6 @@
 
 #include "Decompressor.hpp"
 
-#include "common/MemoryBuffer.hpp"
-
 namespace ancient::internal
 {
 
@@ -15,17 +13,16 @@ class DMSDecompressor : public Decompressor
 {
 public:
 	DMSDecompressor(const Buffer &packedData,bool verify);
+	~DMSDecompressor() noexcept=default;
 
-	virtual ~DMSDecompressor();
+	const std::string &getName() const noexcept final;
+	size_t getPackedSize() const noexcept final;
+	size_t getRawSize() const noexcept final;
 
-	virtual const std::string &getName() const noexcept override final;
-	virtual size_t getPackedSize() const noexcept override final;
-	virtual size_t getRawSize() const noexcept override final;
+	size_t getImageSize() const noexcept final;
+	size_t getImageOffset() const noexcept final;
 
-	virtual size_t getImageSize() const noexcept override final;
-	virtual size_t getImageOffset() const noexcept override final;
-
-	virtual void decompressImpl(Buffer &rawData,bool verify) override final;
+	void decompressImpl(Buffer &rawData,bool verify) final;
 
 	static bool detectHeader(uint32_t hdr) noexcept;
 	static std::shared_ptr<Decompressor> create(const Buffer &packedData,bool exactSizeKnown,bool verify);
@@ -33,17 +30,12 @@ public:
 private:
 	void decompressImpl(Buffer &rawData,bool verify,uint32_t &restartPosition);
 
-	class ShortInputError : public Error
-	{
-		// nothing needed
-	};
-
 	const Buffer	&_packedData;
 
-	uint32_t	_packedSize=0;
-	uint32_t	_rawSize=0;
-	uint32_t	_contextBufferSize=0;
-	uint32_t	_tmpBufferSize=0;
+	uint32_t	_packedSize{0};
+	uint32_t	_rawSize{0};
+	uint32_t	_contextBufferSize{0};
+	uint32_t	_tmpBufferSize{0};
 	uint32_t	_imageSize;
 	uint32_t	_rawOffset;
 	uint32_t	_minTrack;
