@@ -6,15 +6,22 @@
 #include <vector>
 
 #include "BZIP2Decompressor.hpp"
+#include "CompactDecompressor.hpp"
+#include "CompressDecompressor.hpp"
 #include "CRMDecompressor.hpp"
 #include "DEFLATEDecompressor.hpp"
 #include "DMSDecompressor.hpp"
+#include "FreezeDecompressor.hpp"
 #include "IMPDecompressor.hpp"
+#include "LOBDecompressor.hpp"
 #include "MMCMPDecompressor.hpp"
+#include "PackDecompressor.hpp"
 #include "PPDecompressor.hpp"
 #include "RNCDecompressor.hpp"
+#include "SCOCompressDecompressor.hpp"
 #include "StoneCrackerDecompressor.hpp"
 #include "TPWMDecompressor.hpp"
+#include "VicXDecompressor.hpp"
 #include "XPKMain.hpp"
 
 namespace ancient::internal
@@ -24,32 +31,31 @@ namespace ancient::internal
 
 static std::vector<std::pair<bool(*)(uint32_t),std::shared_ptr<Decompressor>(*)(const Buffer&,bool,bool)>> decompressors={
 	{BZIP2Decompressor::detectHeader,BZIP2Decompressor::create},
+	{CompactDecompressor::detectHeader,CompactDecompressor::create},
+	{CompressDecompressor::detectHeader,CompressDecompressor::create},
 	{CRMDecompressor::detectHeader,CRMDecompressor::create},
 	{DEFLATEDecompressor::detectHeader,DEFLATEDecompressor::create},
 	{DMSDecompressor::detectHeader,DMSDecompressor::create},
+	{FreezeDecompressor::detectHeader,FreezeDecompressor::create},
 	{IMPDecompressor::detectHeader,IMPDecompressor::create},
+	{LOBDecompressor::detectHeader,LOBDecompressor::create},
 	{MMCMPDecompressor::detectHeader,MMCMPDecompressor::create},
+	{PackDecompressor::detectHeader,PackDecompressor::create},
 	{PPDecompressor::detectHeader,PPDecompressor::create},
 	{RNCDecompressor::detectHeader,RNCDecompressor::create},
-	{StoneCrackerDecompressor::detectHeader,StoneCrackerDecompressor::create},
+	{SCOCompressDecompressor::detectHeader,SCOCompressDecompressor::create},
 	{TPWMDecompressor::detectHeader,TPWMDecompressor::create},
-	{XPKMain::detectHeader,XPKMain::create}};
-
-Decompressor::Decompressor() noexcept
-{
-	// nothing needed
-}
-
-Decompressor::~Decompressor()
-{
-	// nothing needed
-}
+	{VicXDecompressor::detectHeader,VicXDecompressor::create},
+	{XPKMain::detectHeader,XPKMain::create},
+	// Putting StoneCracker last since detection can be accidentally be detected instead of correct format
+	{StoneCrackerDecompressor::detectHeader,StoneCrackerDecompressor::create}
+	};
 
 std::shared_ptr<Decompressor> Decompressor::create(const Buffer &packedData,bool exactSizeKnown,bool verify)
 {
 	try
 	{
-		uint32_t hdr=packedData.readBE32(0);
+		uint32_t hdr{(packedData.size()>=4)?packedData.readBE32(0):(uint32_t(packedData.readBE16(0))<<16)};
 		for (auto &it : decompressors)
 		{
 			if (it.first(hdr)) return it.second(packedData,exactSizeKnown,verify);
@@ -62,9 +68,10 @@ std::shared_ptr<Decompressor> Decompressor::create(const Buffer &packedData,bool
 
 bool Decompressor::detect(const Buffer &packedData) noexcept
 {
+	if (packedData.size()<2) return false;
 	try
 	{
-		uint32_t hdr=packedData.readBE32(0);
+		uint32_t hdr{(packedData.size()>=4)?packedData.readBE32(0):(uint32_t(packedData.readBE16(0))<<16)};
 		for (auto &it : decompressors)
 			if (it.first(hdr)) return true;
 		return false;
@@ -95,14 +102,15 @@ size_t Decompressor::getImageOffset() const noexcept
 	return 0;
 }
 
+// 1G should be enough for everyone (this is retro!)
 size_t Decompressor::getMaxPackedSize() noexcept
 {
-	return 0x100'0000U;
+	return 0x4000'0000U;
 }
 
 size_t Decompressor::getMaxRawSize() noexcept
 {
-	return 0x100'0000U;
+	return 0x4000'0000U;
 }
 
 }
