@@ -12,12 +12,11 @@ class LIN1Decompressor : public XPKDecompressor
 {
 public:
 	LIN1Decompressor(uint32_t hdr,uint32_t recursionLevel,const Buffer &packedData,std::shared_ptr<XPKDecompressor::State> &state,bool verify);
+	~LIN1Decompressor() noexcept=default;
 
-	virtual ~LIN1Decompressor();
+	const std::string &getSubName() const noexcept final;
 
-	virtual const std::string &getSubName() const noexcept override final;
-
-	virtual void decompressImpl(Buffer &rawData,const Buffer &previousData,bool verify) override final;
+	void decompressImpl(Buffer &rawData,const Buffer &previousData,bool verify) final;
 
 	static bool detectHeaderXPK(uint32_t hdr) noexcept;
 	static std::shared_ptr<XPKDecompressor> create(uint32_t hdr,uint32_t recursionLevel,const Buffer &packedData,std::shared_ptr<XPKDecompressor::State> &state,bool verify);
@@ -25,7 +24,7 @@ public:
 private:
 	const Buffer	&_packedData;
 
-	uint32_t	_ver=0;
+	uint32_t	_ver{0};
 };
 
 }
