@@ -12,12 +12,11 @@ class LZCBDecompressor : public XPKDecompressor
 {
 public:
 	LZCBDecompressor(uint32_t hdr,uint32_t recursionLevel,const Buffer &packedData,std::shared_ptr<XPKDecompressor::State> &state,bool verify);
+	~LZCBDecompressor() noexcept=default;
 
-	virtual ~LZCBDecompressor();
+	const std::string &getSubName() const noexcept final;
 
-	virtual const std::string &getSubName() const noexcept override final;
-
-	virtual void decompressImpl(Buffer &rawData,const Buffer &previousData,bool verify) override final;
+	void decompressImpl(Buffer &rawData,const Buffer &previousData,bool verify) final;
 
 	static bool detectHeaderXPK(uint32_t hdr) noexcept;
 	static std::shared_ptr<XPKDecompressor> create(uint32_t hdr,uint32_t recursionLevel,const Buffer &packedData,std::shared_ptr<XPKDecompressor::State> &state,bool verify);
