@@ -4,25 +4,31 @@
 #include <cstdlib>
 
 #include <memory>
+#include <new>
 
 #include "MemoryBuffer.hpp"
+#include "OverflowCheck.hpp"
+
+
+namespace ancient::internal
+{
 
 MemoryBuffer::MemoryBuffer(size_t size) :
-	_data(reinterpret_cast<uint8_t*>(std::malloc(size))),
-	_size(size)
+	_data{reinterpret_cast<uint8_t*>(std::malloc(size))},
+	_size{size}
 {
-	if (!_data) throw OutOfMemoryError();
+	if (!_data) throw std::bad_alloc();
 }
 
 MemoryBuffer::MemoryBuffer(const Buffer &src,size_t offset,size_t size) :
-	MemoryBuffer(size)
+	MemoryBuffer{size}
 {
-	if(offset+size>src.size()) throw InvalidOperationError();
+	if(OverflowCheck::sum(offset,size)>src.size()) throw InvalidOperationError();
 	std::memcpy(_data,src.data()+offset,size);
 }
 
 
-MemoryBuffer::~MemoryBuffer()
+MemoryBuffer::~MemoryBuffer() noexcept
 {
 	std::free(_data);
 }
@@ -47,13 +53,25 @@ bool MemoryBuffer::isResizable() const noexcept
 	return true;
 }
 
-void MemoryBuffer::resize(size_t newSize) 
+void MemoryBuffer::resize(size_t newSize)
 {
-	_data=reinterpret_cast<uint8_t*>(std::realloc(_data,newSize));
-	_size=newSize;
-	if (!_data)
+	if (!newSize)
 	{
+		std::free(_data);
+		_data=nullptr;
 		_size=0;
-		throw OutOfMemoryError();
+		return;
 	}
+	uint8_t *newData=reinterpret_cast<uint8_t*>(std::realloc(_data,newSize));
+	if (!newData)
+	{
+		std::free(_data);
+		_data=nullptr;
+		_size=0;
+		throw std::bad_alloc();
+	}
+	_data=newData;
+	_size=newSize;
+}
+
 }
